
# q-expansion of newform 9610.2.a.j, downloaded from the LMFDB on 07 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 9610
weight = 2
poly_data = [-3, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1, 0], [1, 1], [-1, 0], [0, 2], [1, -1], [3, 1], [4, 0], [2, -2], [4, 2], [1, 1], [0, 0], [5, -1], [-6, -2], [-5, -1], [0, 2], [7, 1], [2, -2], [3, 7], [-2, -6], [0, 8], [-6, -6], [-14, 2], [3, -1], [6, 4], [4, 0], [-10, 4], [-4, -4], [4, 0], [-12, 2], [8, 0], [2, -8], [2, -6], [0, -12], [-5, 5], [4, 10], [2, 2], [-6, 8], [-6, 2], [0, 10], [-2, 0], [13, -1], [-3, -7], [-2, 0], [2, -8], [-23, -1], [20, 0], [20, -4], [-10, 0], [-10, 2], [5, -11], [4, -12], [6, -2], [-4, -14], [-5, -15], [10, -4], [-6, 12], [9, 5], [2, -2], [15, -7], [10, 0], [6, -2], [0, 10], [-4, 12], [20, 4], [-8, 12], [20, -2], [15, 1], [22, 6], [1, -11], [-32, -2], [6, -6], [-26, 4], [32, 2], [-8, -10], [10, 10], [2, 0], [-21, 3], [-8, -14], [4, -2], [12, 2], [6, 6], [8, -14], [6, -12], [4, 12], [22, 0], [-28, 4], [-12, 10], [-8, 4], [29, 1], [0, -6], [20, 8], [0, 0], [10, 0], [-17, 1], [-7, 7], [-4, -20], [21, 5], [-2, -4], [25, -3], [-4, 10], [-14, 10], [-15, 7], [18, 6], [-12, 10], [-13, 9], [-12, 4], [5, -11], [8, 0], [14, -8], [-22, 4], [12, 0], [-1, 9], [22, -8], [-5, -3], [10, 6], [0, -2], [-37, -5], [-18, 0], [-6, 4], [18, -2], [8, -14], [-16, 12], [-9, -11], [-12, -16], [16, 12], [34, -4], [-11, -11], [12, -8], [16, -12], [-28, -6], [-29, -3], [-6, -16], [6, 8], [15, 17], [-4, -2], [22, -10], [25, -13], [-1, -5], [-3, 23], [-8, -14], [-6, 6], [13, 13], [38, -8], [11, -13], [13, -7], [-26, -12], [-38, 8], [-18, 4], [9, -17], [18, 8], [28, 10], [6, -24], [11, -5], [-12, -22], [4, -8], [24, 4], [-20, -12], [12, 6], [-26, -4], [39, -9], [5, -3], [-24, 4], [26, 8], [-48, 0], [22, -12], [10, -16], [-20, -4], [8, 18], [0, 10], [-21, -11], [19, -3], [11, -21], [30, -12], [8, 24], [-34, -4], [2, -20], [32, 0], [18, -4], [16, -4], [-29, 11], [-8, -12], [27, 5], [-40, -2], [30, -6], [-32, -14], [11, 23], [10, 28], [36, -8], [8, -26], [52, 4], [-14, 18], [-2, -30], [-7, 15], [45, -3], [-4, 12], [-14, 6], [-2, 28], [-24, -6], [30, -12], [12, 18], [8, -14], [-16, 12], [-2, -20], [18, -14], [-56, -4], [29, 11], [22, -4], [-3, -15], [-30, 2], [4, 24], [52, 8], [5, 5], [40, -2], [6, -22], [-24, -4], [6, 6], [-32, -10], [-46, 10], [-30, 12], [-26, -12], [25, 9], [-12, 20], [-6, -10], [28, -4], [18, 10], [-31, -19], [18, 16], [50, -2], [8, -26], [0, -16], [25, -9], [-28, 16], [14, 8], [18, -24], [23, -9], [32, -18], [-54, 8], [30, -4], [23, -7], [16, -12], [-12, -16], [-13, -23], [-28, 26], [-7, 33], [2, -26], [36, -20], [4, -18], [-9, 5], [43, 17], [24, 24], [-4, -26], [-18, -6], [-16, 6], [6, 42], [38, 8], [-4, -24], [-8, 6], [-23, -11], [-18, 0], [-22, 16], [40, 2], [35, 7], [-9, 3], [-22, 0], [68, 0], [-12, -4], [-32, -6], [-26, 0], [0, -20], [8, 10], [14, 16], [35, 11], [34, -18], [24, 20], [-16, 0], [16, 28], [30, -2], [-35, -3], [-18, -16], [9, -21], [-68, -2], [32, 8], [0, 0], [8, -26], [-4, 20], [-6, -34], [-48, -8], [21, 3], [-18, -4], [2, -24], [6, -36], [-12, 20], [-46, 2], [-16, 4], [25, -13], [-39, 21], [-38, 10], [0, -26], [-5, 5], [33, 25], [-28, 0], [-1, 5], [12, -24], [-42, -10], [-7, 3], [52, -8], [1, 5], [20, 18], [2, -14], [-22, 0], [22, -36], [13, -11], [30, -12], [-23, -27], [-24, -36], [-46, 24], [51, 21], [18, -18], [0, -16], [-66, 8], [1, -33], [-4, -16], [24, -6], [-52, 14], [-84, 0], [-28, -26], [50, -14], [-62, -6], [-18, 16], [1, 35], [-56, -2], [-24, 26], [20, -4], [-9, 23], [4, 36], [46, 2], [31, -9], [-10, -8], [-42, -8], [-16, 26], [-7, 39], [48, 8], [-47, -11], [32, 20], [38, -4], [6, -2], [8, -26], [-21, 23], [26, -34], [30, -16], [27, 5], [28, 0], [36, 26], [-16, -30], [28, -6], [-66, 2], [14, 38], [-21, -7], [-12, -16], [-52, 2], [-54, 20], [-6, -24], [-16, -30], [-6, -14], [42, -14], [-6, -30], [-48, 2], [40, -10], [-6, -20], [38, 6], [-30, -26], [-32, -28], [0, 34], [-30, -32], [-39, 11], [-23, 19], [40, -8], [38, 4], [-18, -24], [-16, -32], [-45, -21], [-20, 32], [-74, 12], [18, -22], [37, -21], [52, -24], [8, 8], [25, 19], [61, -15], [28, 22], [-14, -32], [33, -25], [-46, 6], [64, 4], [72, -8], [64, 12], [-70, 6], [-30, -32], [45, -29], [5, 45], [5, -15], [-58, 4], [40, -16], [-24, 36], [-7, -19], [-26, 4], [-14, -24], [-52, 14], [-35, -15], [-61, 13], [2, -18], [68, 14], [-39, -31], [35, -35], [22, -20], [66, -8], [20, 28], [-12, -50], [-28, -16], [76, 4], [-51, -7], [49, -1], [-18, 32], [14, -6], [26, -12], [53, 3], [18, -38], [54, 12], [-71, -5], [50, 10]]
